#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: ip_make_sense.pl,v 1.0.0.8 Tue 01 Jun 2010 09:54:52 AM CEST mstauber Exp $
# Copyright 2006-2010 Stauber Multimedia Design. All rights reserved.
#
# Run checks on the values passed through the GUI to make sure that they make sense.
# This one is for the IP address allocation of VPS's.

use CCE;
use POSIX qw(isalpha);

my $DEBUG = 0;

my $cce = new CCE('Domain' => 'base-vserver');

$cce->connectfd();
#$cce->connectuds(); # only for debugging

my $vserver = $cce->event_object();
my $vserver_new = $cce->event_new();
my $vserver_old = $cce->event_old();

my @oids= $cce->find("Vserver");
my ($ok, $system) = $cce->get($oids[0]);

# Check if VPS IP-Address causes conflict:
if (($vserver_new->{min}) || ($vserver->{min}) || ($vserver_old->{min})) {

	if ($vserver->{min} ne '') { $validate_ip = $vserver->{min}; }
	if ($vserver_old->{min} ne '') { $validate_ip = $vserver_old->{min}; }
	if ($vserver_new->{min} ne '') { $validate_ip = $vserver_new->{min}; }

        my @oids = $cce->findx("Vserver", {},
                               {
                                'ip' => $validate_ip,
                                'is_dir_present' => '1'
                               });

        # There should be no oids found   
        #
        # Unfortunately both find() and findx() are stupid and do partial matches.
        # So 127.0.0.1 also matches 127.0.0.10 :o(
        # Therefore we have to do some extra magic:
        if (scalar(@oids) > 1) {
                $the_scalar = scalar(@oids);
                $out_msg = "";   
                # Walk through all returned oids. One of them is the one we want to modify. Any extra result may
                # be a partial match.
                foreach $entry (@oids) {
                        ($ok, $conflict) = $cce->get($entry);
                        # Check if a returned match has the same hostname. If so, it's the object we want to write to and which we then ignore.
                        # Compare with 'eq' to find out if it's a fully *identical* IP:
                        if (($conflict->{ip} eq $validate_ip) && ($vserver->{hostname} ne $conflict->{hostname})) {
                                $out_msg = $out_msg . $entry . " [" . $conflict->{ip} . " == " . $validate_ip . "] ";
                                # Ok, returned result is identical AND found in a VPS with differrent hostname.
                                # So we - rightfully - throw an error:  
                                $cce->bye('FAIL',
                                        "[[base-vserver.ipInUse,ip='$validate_ip (CCE Object ID: $out_msg - $the_scalar)']]");
        
                                exit(1);
                        }
                }
        }

	# verify that the VPS doesn't have the same IP as the master node itself:
	my ($network) = $cce->find('Network', { 'ipaddr' => $validate_ip, 'enabled' => '1' });
        if ($network) {
                $cce->bye('FAIL', "[[base-vserver.ipInUseMasterNode,ip='$validate_ip']]");
                exit(1);
	}

	# Now check if this IP has already been assigned as extra IP for another VPS:
        my @oids = $cce->findx('vpsIPalloc', { 'min' => $validate_ip });
        # there should be no oids found
        if (scalar(@oids) > 1) {
		print "FAIL! \n";
                $cce->bye('FAIL', "[[base-vserver.ipInUseExtra,ip='$validate_ip']]");
                exit(1);
        }
}


$cce->bye('SUCCESS');
exit(0);

